
`default_nettype none

module top(
	input  wire Reset,				// 17	
	input  wire Play_i,				// 66
	input  wire Stop_i,				// 69
	output wire SoundWave_o,		// 28
	
	output wire [7:0] Segments_o,
	output wire [7:0] Cathodes_o
);

	// Generator sygnau zegarowego
	parameter CLOCK_HZ = 14_000_000;
	wire Clock;
	OSCH #(
		.NOM_FREQ("14.00")
	) OSCH_inst(
		.STDBY(1'b0),
		.OSC(Clock),
		.SEDSTDBY()
	);
	
	// Wykrywanie zbocza opadajcego na przycisku enkodera E41
	// W stanie spoczynkowym jest stan 1
	wire Play;
	
	Encoder EncoderPlay(
		.Clock(Clock),
		.Reset(Reset),
		.AsyncA_i(1'b1),
		.AsyncB_i(1'b1),
		.AsyncS_i(Play_i),
		.Increment_o(),
		.Decrement_o(),
		.ButtonPress_o(Play),
		.ButtonRelease_o(),
		.ButtonState_o()		
	);
	
	wire Stop;
	
	Encoder EncoderStop(
		.Clock(Clock),
		.Reset(Reset),
		.AsyncA_i(1'b1),
		.AsyncB_i(1'b1),
		.AsyncS_i(Stop_i),
		.Increment_o(),
		.Decrement_o(),
		.ButtonPress_o(Stop),
		.ButtonRelease_o(),
		.ButtonState_o()		
	);		
	
	// Wire for debug on the display
	wire [15:0] Duration_ms;
	wire [15:0] HalfPeriod_us;
	
	// Instantiate device under test
	MelodyPlayer #(
		.CLOCK_HZ(CLOCK_HZ)
	) MelodyPlayer_inst(
		.Clock(Clock),
		.Reset(Reset),
		.Play_i(Play),
		.Stop_i(Stop),
		.SoundWave_o(SoundWave_o),
		.Duration_o(Duration_ms),
		.HalfPeriod_o(HalfPeriod_us)
	);
	
	// Display instance
	DisplayMultiplex #(
		.CLOCK_HZ(CLOCK_HZ),
		.SWITCH_PERIOD_US(1000), // max 2000
		.DIGITS(8)
	) DisplayMultiplex_inst(
		.Clock(Clock),
		.Reset(Reset),
		.Data_i({Duration_ms, HalfPeriod_us}),
		.DecimalPoints_i(8'b00010000),
		.Cathodes_o(Cathodes_o),
		.Segments_o(Segments_o),
		.SwitchCathode_o()
	);

endmodule

`default_nettype wire
